@@ -2,14 +2,16 @@
 /**
  * @file defLoad.c
  *
- *  Time-stamp:        "2012-03-04 11:02:51 bkorb"
- *
  *  This module loads the definitions, calls yyparse to decipher them,
  *  and then makes a fixup pass to point all children definitions to
  *  their parent definition.
  *
+ * @addtogroup autogen
+ * @{
+ */
+/*
  *  This file is part of AutoGen.
- *  AutoGen Copyright (c) 1992-2012 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (C) 1992-2015 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -31,17 +33,20 @@ typedef enum {
     INPUT_FILE
 } def_input_mode_t;
 
-static def_ent_t* pFreeEntryList = NULL;
-static void*      pAllocList     = NULL;
+static def_ent_t * free_de_list = NULL;
+static void *      de_blocks    = NULL;
 
-#define ENTRY_SPACE        (4096 - sizeof(void*))
+#define ENTRY_SPACE        (4096 - sizeof(void *))
 #define ENTRY_ALLOC_CT     (ENTRY_SPACE / sizeof(def_ent_t))
 #define ENTRY_ALLOC_SIZE   \
-    ((ENTRY_ALLOC_CT * sizeof(def_ent_t)) + sizeof(void*))
+    ((ENTRY_ALLOC_CT * sizeof(def_ent_t)) + sizeof(void *))
 
 /* = = = START-STATIC-FORWARD = = = */
-static def_ent_t*
-insert_ent(def_ent_t* pDef);
+static void
+free_def_ent(def_ent_t * de);
+
+static def_ent_t *
+insert_ent(def_ent_t * de);
 
 static def_input_mode_t
 ready_def_input(char const ** ppzfile, size_t * psz);
@@ -50,32 +55,32 @@ ready_def_input(char const ** ppzfile, size_t * psz);
 LOCAL def_ent_t *
 new_def_ent(void)
 {
-    def_ent_t * pRes = pFreeEntryList;
+    def_ent_t * res = free_de_list;
 
-    if (pRes != NULL) {
-        pFreeEntryList = pRes->de_next;
+    if (res != NULL) {
+        free_de_list = res->de_next;
 
     } else {
-        int   ct = ENTRY_ALLOC_CT-1;
-        void* p  = AGALOC(ENTRY_ALLOC_SIZE, "def headers");
+        int    ct = ENTRY_ALLOC_CT-1;
+        void * p  = AGALOC(ENTRY_ALLOC_SIZE, "def headers");
 
-        *((void**)p) = pAllocList;
-        pAllocList   = p;
-        pRes = pFreeEntryList = (def_ent_t*)((void**)p + 1);
+        *((void **)p) = de_blocks;
+        de_blocks     = p;
+        res = free_de_list = (def_ent_t *)((void **)p + 1);
 
         /*
          *  This is a post-loop test loop.  It will cycle one fewer times
          *  than there are 'def_ent_t' structs in the memory we just alloced.
          */
         do  {
-            def_ent_t* pNxt = pRes+1;
-            pRes->de_next = pNxt;
+            def_ent_t * next = res+1;
+            res->de_next = next;
 
             /*
-             *  When the loop ends, "pRes" will point to the last allocated
+             *  When the loop ends, "res" will point to the last allocated
              *  structure instance.  That is the one we will return.
              */
-            pRes = pNxt;
+            res = next;
         } while (--ct > 0);
 
         /*
@@ -83,25 +88,33 @@ new_def_ent(void)
          *  routine is called, the *FIRST* structure in this list will
          *  be returned.
          */
-        pRes[-1].de_next = NULL;
+        res[-1].de_next = NULL;
     }
 
-    memset((void*)pRes, 0, sizeof(*pRes));
-    return pRes;
+    memset(VOIDP(res), 0, sizeof(*res));
+    return res;
 }
 
-/*
- *  Append a new entry at the end of a sibling (or twin) list.
+static void
+free_def_ent(def_ent_t * de)
+{
+    de->de_next  = free_de_list;
+    free_de_list = de;
+}
+
+/**
+ * Append a new entry at the end of a sibling (or twin) list.
+ * @param de  new definition
  */
 LOCAL void
-print_ent(def_ent_t * pDef)
+print_ent(def_ent_t * de)
 {
     int ix = 32 - (2 * ent_stack_depth);
     char const * space = PRINT_DEF_SPACES + ((ix < 0) ? 0 : ix);
 
     char const * vtyp;
 
-    switch (pDef->de_type) {
+    switch (de->de_type) {
     case VALTYP_UNKNOWN: vtyp = DEF_TYPE_UNKNOWN; break;
     case VALTYP_TEXT:    vtyp = DEF_TYPE_TEXT;    break;
     case VALTYP_BLOCK:   vtyp = DEF_TYPE_BLOCK;   break;
@@ -110,132 +123,204 @@ print_ent(def_ent_t * pDef)
 
     fprintf(trace_fp, PRINT_DEF_SHOW_FMT,
             space,
-            pDef->de_name, (unsigned int)pDef->de_index,
+            de->de_name, (unsigned int)de->de_index,
             vtyp,
-            pDef->de_file, pDef->de_line);
+            de->de_file, de->de_line, de);
+}
+
+/**
+ *  Remove a new entry from a sibling (or twin) list.
+ *
+ * @param[in]  de  dead definition
+ */
+LOCAL void
+delete_ent(def_ent_t * de)
+{
+    def_ent_t * de_list = ent_stack[ ent_stack_depth ];
+    def_ent_t ** list_p = &(de_list->de_val.dvu_entry);
+
+    if (*list_p == NULL)
+        AG_ABEND(RM_MISSING_DE);
+
+    de_list = de_list->de_val.dvu_entry;
+
+    while (strcmp(de->de_name, de_list->de_name) != 0) {
+        if (de_list->de_next == NULL)
+            AG_ABEND(RM_MISSING_DE);
+        list_p  = &(de_list->de_next);
+        de_list = de_list->de_next;
+    }
+
+    /*
+     * Check for single entry list.  I.e. point the single pointer to
+     * this entry to the next, possibly NULL, entry.
+     */
+    if (de_list->de_etwin == NULL) {
+        if (de_list != de)
+            AG_ABEND(RM_MISSING_DE);
+        *list_p = de->de_next;
+    }
+
+    /*
+     * Multiple entry list.  Check for list head.
+     */
+    else if (de_list == de) {
+        /*
+         * The list head now becomes the second "twin".
+         * Adjust all the links *except* de_twin.
+         */
+        de_list = *list_p = de->de_twin;
+        de_list->de_etwin = de->de_etwin;
+        de_list->de_ptwin = NULL;
+        de_list->de_next  = de->de_next;
+    }
+
+    /*
+     * Somewhere in the middle.  Fix up the de_twin and de_ptwin pointers
+     * around the current entry.  If the current entry is actually last,
+     * then fix up the de_etwin pointer from the twin list head.
+     */
+    else {
+        def_ent_t * pde = de->de_ptwin;
+        def_ent_t * nde = de->de_twin;
+        pde->de_twin = de->de_twin;
+        if (nde != NULL)
+            nde->de_ptwin = pde;
+        else if (de_list->de_etwin != de)
+            AG_ABEND(RM_MISSING_DE);
+        else
+            de_list->de_etwin = pde;
+    }
+
+    free_def_ent(de);
 }
 
 /**
  *  Append a new entry into a sibling (or twin) list.
  *
- * @param pDef new definition
+ * @param[in]  de new definition
  * @returns usually, the input, but sometimes it is necessary to move
  *  the data, so returns the address of the incoming data regardless.
  */
-static def_ent_t*
-insert_ent(def_ent_t* pDef)
+static def_ent_t *
+insert_ent(def_ent_t * de)
 {
-    def_ent_t* pList = ent_stack[ ent_stack_depth ];
+    def_ent_t * de_list = ent_stack[ ent_stack_depth ];
 
     /*
      *  If the current level is empty, then just insert this one and quit.
      */
-    if (pList->de_val.dvu_entry == NULL) {
-        if (pDef->de_index == NO_INDEX)
-            pDef->de_index = 0;
-        pList->de_val.dvu_entry = pDef;
+    if (de_list->de_val.dvu_entry == NULL) {
+        if (de->de_index == NO_INDEX)
+            de->de_index = 0;
+        de_list->de_val.dvu_entry = de;
 
-        return pDef;
+        return de;
     }
-    pList = pList->de_val.dvu_entry;
+    de_list = de_list->de_val.dvu_entry;
 
     /*
      *  Scan the list looking for a "twin" (same-named entry).
      */
-    while (strcmp(pDef->de_name, pList->de_name) != 0) {
+    while (strcmp(de->de_name, de_list->de_name) != 0) {
         /*
          *  IF we are at the end of the list,
          *  THEN put the new entry at the end of the list.
          *       This is a new name in the current context.
          *       The value type is forced to be the same type.
          */
-        if (pList->de_next == NULL) {
-            pList->de_next = pDef;
+        if (de_list->de_next == NULL) {
+            de_list->de_next = de;
 
-            if (pDef->de_index == NO_INDEX)
-                pDef->de_index = 0;
+            if (de->de_index == NO_INDEX)
+                de->de_index = 0;
 
-            return pDef;
+            return de;
         }
 
         /*
          *  Check the next sibling for a twin value.
          */
-        pList = pList->de_next;
+        de_list = de_list->de_next;
     }
 
     /*  * * * * *  WE HAVE FOUND A TWIN
      *
      *  Link in the new twin chain entry into the list.
      */
-    if (pDef->de_index == NO_INDEX) {
-        def_ent_t* pT = pList->de_etwin;
+    if (de->de_index == NO_INDEX) {
+        def_ent_t * pT = de_list->de_etwin;
         if (pT == NULL)
-            pT = pList;
+            pT = de_list;
 
-        pDef->de_index  = pT->de_index + 1;
-        pT->de_twin     = pDef;
-        pDef->de_ptwin  = pT;
-        pList->de_etwin = pDef;
+        de->de_index = pT->de_index + 1;
+        pT->de_twin  = de;
+        de->de_ptwin = pT;
+        de_list->de_etwin = de;
 
-    } else if (pList->de_index > pDef->de_index) {
+    } else if (de_list->de_index > de->de_index) {
 
         /*
-         *  Insert the new entry before any other in the list.
-         *  We actually do this by leaving the pList pointer alone and swapping
-         *  the contents of the definition entry.
+         *  Insert the new entry before any other in the list.  We
+         *  actually do this by leaving the de_list pointer alone and
+         *  swapping the contents of the definition entry.
          */
-        def_ent_t def = *pDef;
+        def_ent_t def = *de;
 
-        memcpy(&(pDef->de_name), &(pList->de_name),
+        memcpy(&(de->de_name), &(de_list->de_name),
                sizeof(def) - ag_offsetof(def_ent_t, de_name));
 
-        memcpy(&(pList->de_name), &(def.de_name),
+        memcpy(&(de_list->de_name), &(def.de_name),
                sizeof(def) - ag_offsetof(def_ent_t, de_name));
 
         /*
-         * Contents are swapped.  Link "pDef" after "pList" and return "pList".
+         * Contents are swapped.  Link "de" after "de_list" and return "de_list".
          */
-        pDef->de_twin = pList->de_twin;
-        if (pDef->de_twin != NULL)
-            pDef->de_twin->de_ptwin = pDef;
+        de->de_twin = de_list->de_twin;
+        if (de->de_twin != NULL)
+            de->de_twin->de_ptwin = de;
 
-        pDef->de_ptwin = pList;
-        pList->de_twin  = pDef;
+        de->de_ptwin = de_list;
+        de_list->de_twin  = de;
 
         /*
          *  IF this is the first twin, then the original list head is now
          *  the "end twin".
          */
-        if (pList->de_etwin == NULL)
-            pList->de_etwin = pDef;
+        if (de_list->de_etwin == NULL)
+            de_list->de_etwin = de;
 
-        pDef = pList;  /* Return the replacement structure address */
+        de = de_list;  /* Return the replacement structure address */
 
     } else {
-        def_ent_t* pL = pList;
-        def_ent_t* pT = pL->de_twin;
+        def_ent_t * scn = de_list;
+        def_ent_t * twn = scn->de_twin;
 
         /*
          *  Insert someplace after the first entry.  Scan the list until
          *  we either find a larger index or we get to the end.
          */
-        while ((pT != NULL) && (pT->de_index < pDef->de_index)) {
-            pL = pT;
-            pT = pT->de_twin;
+        while (twn != NULL) {
+            if (twn->de_index >= de->de_index) {
+                if (twn->de_index == de->de_index)
+                    AG_ABEND(aprf(DUP_VALUE_INDEX, de->de_name, de->de_index));
+                break;
+            }
+
+            scn = twn;
+            twn = twn->de_twin;
         }
 
-        pDef->de_twin = pT;
-
-        pL->de_twin = pDef;
-        pDef->de_ptwin = pL;
-        if (pT == NULL)
-            pList->de_etwin = pDef;
+        de->de_twin  = twn;
+        scn->de_twin = de;
+        de->de_ptwin = scn;
+        if (twn == NULL)
+            de_list->de_etwin = de;
         else
-            pT->de_ptwin = pDef;
+            twn->de_ptwin = de;
     }
 
-    return pDef; /* sometimes will change */
+    return de; /* sometimes will change */
 }
 
 /**
@@ -263,7 +348,7 @@ number_and_insert_ent(char * name, char const * idx_str)
 
     strtransform(ent->de_name, ent->de_name);
     ent->de_type  = VALTYP_UNKNOWN;
-    ent->de_file  = (char*)cctx->scx_fname;
+    ent->de_file  = (char *)cctx->scx_fname;
     ent->de_line  = cctx->scx_line;
     return (curr_ent = insert_ent(ent));
 }
@@ -277,8 +362,8 @@ ready_def_input(char const ** ppzfile, size_t * psz)
     struct stat stbf;
 
     if (! ENABLED_OPT(DEFINITIONS)) {
-        base_ctx = (scan_ctx_t*)AGALOC(sizeof(scan_ctx_t), "scan context");
-        memset((void*)base_ctx, 0, sizeof(scan_ctx_t));
+        base_ctx = (scan_ctx_t *)AGALOC(sizeof(scan_ctx_t), "scan context");
+        memset(VOIDP(base_ctx), 0, sizeof(scan_ctx_t));
         base_ctx->scx_line  = 1;
         base_ctx->scx_fname = READY_INPUT_NODEF;
 
@@ -308,7 +393,7 @@ ready_def_input(char const ** ppzfile, size_t * psz)
         }
 
     accept_fifo:
-        outfile_time  = time(NULL);
+        maxfile_time = outfile_time = time(NULL);
         *psz = 0x4000 - (4+sizeof(*base_ctx));
         return INPUT_STDIN;
     }
@@ -334,8 +419,9 @@ ready_def_input(char const ** ppzfile, size_t * psz)
      *  the mod time on this file.  If any of the template files
      *  are more recent, then it will be adjusted.
      */
-    *psz = stbf.st_size;
+    *psz = (size_t)stbf.st_size;
 
+    maxfile_time = stbf.st_mtime;
     outfile_time = ENABLED_OPT(SOURCE_TIME) ? stbf.st_mtime : time(NULL);
 
     return INPUT_FILE;
@@ -361,8 +447,8 @@ read_defs(void)
      *  Allocate the space we need for our definitions.
      */
     sizeLeft = dataSize+4+sizeof(*base_ctx);
-    base_ctx = (scan_ctx_t*)AGALOC(sizeLeft, "file buf");
-    memset((void*)base_ctx, 0, sizeLeft);
+    base_ctx = (scan_ctx_t *)AGALOC(sizeLeft, "file buf");
+    memset(VOIDP(base_ctx), 0, sizeLeft);
     base_ctx->scx_line = 1;
     sizeLeft = dataSize;
 
@@ -374,7 +460,7 @@ read_defs(void)
      */
     pzData =
         base_ctx->scx_scan =
-        base_ctx->scx_data = (char*)(base_ctx+1);
+        base_ctx->scx_data = (char *)(base_ctx + 1);
     base_ctx->scx_next     = NULL;
 
     /*
@@ -396,7 +482,7 @@ read_defs(void)
      *  Read until done...
      */
     for (;;) {
-        size_t rdct = fread((void*)pzData, (size_t)1, sizeLeft, fp);
+        size_t rdct = fread(VOIDP(pzData), (size_t)1, sizeLeft, fp);
 
         /*
          *  IF we are done,
@@ -422,7 +508,7 @@ read_defs(void)
          *  See if there is any space left
          */
         if (sizeLeft == 0) {
-            scan_ctx_t* p;
+            scan_ctx_t * p;
             off_t dataOff;
 
             /*
@@ -437,7 +523,7 @@ read_defs(void)
              */
             dataSize += (sizeLeft = 0x1000);
             dataOff = pzData - base_ctx->scx_data;
-            p = AGREALOC((void*)base_ctx, dataSize+4+sizeof(*base_ctx),
+            p = AGREALOC(VOIDP(base_ctx), dataSize + 4 + sizeof(*base_ctx),
                          "expand f buf");
 
             /*
@@ -447,7 +533,7 @@ read_defs(void)
              */
             if (p != base_ctx) {
                 p->scx_scan = \
-                    p->scx_data = (char*)(p+1);
+                    p->scx_data = (char *)(p + 1);
                 pzData = p->scx_data + dataOff;
                 base_ctx = p;
             }
@@ -477,7 +563,9 @@ unload_defs(void)
 {
     return;
 }
-/*
+/**
+ * @}
+ *
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
