@@ -10,7 +10,7 @@
  */
 /*
  *  This file is part of AutoGen.
- *  AutoGen Copyright (C) 1992-2013 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (C) 1992-2015 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -110,10 +110,10 @@ stringify_for_sh(char * pzNew, uint_t qt, char const * pzDta)
 static SCM
 shell_stringify(SCM obj, uint_t qt)
 {
-    char*  pzNew;
+    char * pzNew;
     size_t dtaSize = 3;
-    char*  pzDta   = ag_scm2zchars(obj, "AG Object");
-    char*  pz      = pzDta;
+    char * pzDta   = ag_scm2zchars(obj, "AG Object");
+    char * pz      = pzDta;
 
     for (;;) {
         char c = *(pz++);
@@ -178,10 +178,10 @@ do_substitution(
     char **      ppz_res,
     ssize_t *    res_len)
 {
-    char* pzMatch  = ag_scm2zchars(match, "match text");
-    char* rep_str  = ag_scm2zchars(repl,  "repl text");
-    int   mark_len = (int)AG_SCM_STRLEN(match);
-    int   repl_len = (int)AG_SCM_STRLEN(repl);
+    char * pzMatch  = ag_scm2zchars(match, "match text");
+    char * rep_str  = ag_scm2zchars(repl,  "repl text");
+    int    mark_len = (int)AG_SCM_STRLEN(match);
+    int    repl_len = (int)AG_SCM_STRLEN(repl);
 
     {
         int ct = sub_count(src_str, pzMatch);
@@ -226,13 +226,13 @@ do_substitution(
 LOCAL void
 do_multi_subs(char ** ppzStr, ssize_t * pStrLen, SCM match, SCM repl)
 {
-    char* pzStr = *ppzStr;
-    char* pzNxt = pzStr;
+    char * pzStr = *ppzStr;
+    char * pzNxt = pzStr;
 
     /*
      *  Loop for as long as our list has more entries
      */
-    while (! AG_SCM_NULLP(match)) {
+    while (! scm_is_null(match)) {
         /*
          *  "CAR" is the current value, "CDR" is rest of list
          */
@@ -306,7 +306,7 @@ ag_scm_in_p(SCM obj, SCM list)
     if (! AG_SCM_STRING_P(obj))
         return SCM_UNDEFINED;
 
-    pz1  = AG_SCM_CHARS(obj);
+    pz1  = scm_i_string_chars(obj);
     lenz = AG_SCM_STRLEN(obj);
 
     /*
@@ -315,7 +315,7 @@ ag_scm_in_p(SCM obj, SCM list)
      */
     if (AG_SCM_STRING_P(list)) {
         if (  (AG_SCM_STRLEN(list) == lenz)
-           && (strncmp(pz1, AG_SCM_CHARS(list), lenz) == 0))
+           && (strncmp(pz1, scm_i_string_chars(list), lenz) == 0))
             return SCM_BOOL_T;
         return SCM_BOOL_F;
     }
@@ -344,7 +344,7 @@ ag_scm_in_p(SCM obj, SCM list)
         }
 
         if (  (AG_SCM_STRLEN(car) == lenz)
-           && (strncmp(pz1, AG_SCM_CHARS(car), lenz) == 0) )
+           && (strncmp(pz1, scm_i_string_chars(car), lenz) == 0) )
             return SCM_BOOL_T;
     }
 
@@ -383,7 +383,7 @@ ag_scm_join(SCM sep, SCM list)
     if (l_len == 0)
         return AG_SCM_STR02SCM(zNil);
 
-    pzSep   = AG_SCM_CHARS(sep);
+    pzSep   = scm_i_string_chars(sep);
     sep_len = AG_SCM_STRLEN(sep);
     str_len = 0;
 
@@ -434,7 +434,7 @@ ag_scm_join(SCM sep, SCM list)
             car = ag_scm_join(sep, car);
 
         cpy_len = AG_SCM_STRLEN(car);
-        memcpy((void*)pzScan, AG_SCM_CHARS(car), cpy_len);
+        memcpy(VOIDP(pzScan), scm_i_string_chars(car), cpy_len);
         pzScan += cpy_len;
 
         /*
@@ -442,7 +442,7 @@ ag_scm_join(SCM sep, SCM list)
          */
         if (--l_len <= 0)
             break;
-        memcpy((void*)pzScan, (void*)pzSep, sep_len);
+        memcpy(VOIDP(pzScan), VOIDP(pzSep), sep_len);
         pzScan += sep_len;
     }
 
@@ -733,11 +733,11 @@ ag_scm_sub_shell_str(SCM obj)
 SCM
 ag_scm_stack(SCM obj)
 {
-    SCM         res;
-    SCM *       pos = &res;
-    def_ent_t** ppDE;
-    def_ent_t*  pDE;
-    SCM         str;
+    SCM          res;
+    SCM *        pos = &res;
+    def_ent_t ** ppDE;
+    def_ent_t *  pDE;
+    SCM          str;
 
     res = SCM_EOL;
 
@@ -841,9 +841,9 @@ SCM
 ag_scm_string_tr_x(SCM str, SCM from_xform, SCM to_xform)
 {
     unsigned char ch_map[ 1 << 8 /* bits-per-byte */ ];
-    int   i    = sizeof(ch_map) - 1;
-    char* from = ag_scm2zchars(from_xform, "str");
-    char* to   = ag_scm2zchars(to_xform, "str");
+    int    i    = sizeof(ch_map) - 1;
+    char * from = ag_scm2zchars(from_xform, "str");
+    char * to   = ag_scm2zchars(to_xform, "str");
 
     do  {
         ch_map[i] = (unsigned char)i;
@@ -882,7 +882,7 @@ ag_scm_string_tr_x(SCM str, SCM from_xform, SCM to_xform)
         }
     } map_done:;
 
-    to = C(char *, AG_SCM_CHARS(str));
+    to = C(char *, scm_i_string_chars(str));
     i    = (int)AG_SCM_STRLEN(str);
     while (i-- > 0) {
         *to = (char)ch_map[ (int)*to ];
@@ -907,7 +907,7 @@ SCM
 ag_scm_string_tr(SCM Str, SCM From, SCM To)
 {
     size_t lenz  = AG_SCM_STRLEN(Str);
-    SCM    res   = AG_SCM_STR2SCM(AG_SCM_CHARS(Str), lenz);
+    SCM    res   = AG_SCM_STR2SCM(scm_i_string_chars(Str), lenz);
     return ag_scm_string_tr_x(res, From, To);
 }
 
@@ -941,13 +941,13 @@ ag_scm_string_substitute(SCM str, SCM Match, SCM Repl)
     if (! AG_SCM_STRING_P(str))
         return SCM_UNDEFINED;
 
-    text = AG_SCM_CHARS(str);
+    text = scm_i_string_chars(str);
     len   = (ssize_t)AG_SCM_STRLEN(str);
 
     if (AG_SCM_STRING_P(Match))
-        do_substitution(text, len, Match, Repl, (char**)&text, &len);
+        do_substitution(text, len, Match, Repl, (char **)&text, &len);
     else
-        do_multi_subs((char**)&text, &len, Match, Repl);
+        do_multi_subs((char **)&text, &len, Match, Repl);
 
     res = AG_SCM_STR2SCM(text, (size_t)len);
     return res;
@@ -977,10 +977,10 @@ ag_scm_time_string_to_number(SCM time_spec)
     if (! AG_SCM_STRING_P(time_spec))
         return SCM_UNDEFINED;
 
-    pz = AG_SCM_CHARS(time_spec);
+    pz = scm_i_string_chars(time_spec);
     time_period = parse_duration(pz);
 
-    return AG_SCM_INT2SCM((int)time_period);
+    return scm_from_int((int)time_period);
 }
 
 /**
